@@ -1,4 +1,4 @@
-// Copyright 2020 The Grin Developers
+// Copyright 2021 The Grin Developers
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -69,6 +69,29 @@ impl Writeable for SegmentIdentifier {
 	}
 }
 
+impl SegmentIdentifier {
+	/// Test helper to get an iterator of SegmentIdentifiers required to read a
+	/// pmmr of size `target_mmr_size` in segments of height `segment_height`
+	pub fn traversal_iter(
+		target_mmr_size: u64,
+		segment_height: u8,
+	) -> impl Iterator<Item = SegmentIdentifier> {
+		(0..SegmentIdentifier::count_segments_required(target_mmr_size, segment_height)).map(
+			move |idx| SegmentIdentifier {
+				height: segment_height,
+				idx: idx as u64,
+			},
+		)
+	}
+
+	/// Returns number of segments required that would needed in order to read a
+	/// pmmr of size `target_mmr_size` in segments of height `segment_height`
+	pub fn count_segments_required(target_mmr_size: u64, segment_height: u8) -> usize {
+		let d = 1 << segment_height;
+		((pmmr::n_leaves(target_mmr_size) + d - 1) / d) as usize
+	}
+}
+
 /// Segment of a PMMR: unpruned leaves and the necessary data to verify
 /// segment membership in the original MMR.
 #[derive(Clone, Debug, Eq, PartialEq)]
@@ -104,42 +127,41 @@ impl<T> Segment<T> {
 		self.identifier.idx * self.segment_capacity()
 	}
 
-	/// Number of leaves in this segment. Equal to capacity except for the final segment, which can be smaller
-	fn segment_unpruned_size(&self, last_pos: u64) -> u64 {
+	// Number of leaves in this segment. Equal to capacity except for the final segment, which can be smaller
+	fn segment_unpruned_size(&self, mmr_size: u64) -> u64 {
 		min(
 			self.segment_capacity(),
-			pmmr::n_leaves(last_pos).saturating_sub(self.leaf_offset()),
+			pmmr::n_leaves(mmr_size).saturating_sub(self.leaf_offset()),
 		)
 	}
 
-	/// Whether the segment is full (size == capacity)
-	fn full_segment(&self, last_pos: u64) -> bool {
-		self.segment_unpruned_size(last_pos) == self.segment_capacity()
+	/// Whether the segment is full (segment size == capacity)
+	fn full_segment(&self, mmr_size: u64) -> bool {
+		self.segment_unpruned_size(mmr_size) == self.segment_capacity()
 	}
 
 	/// Inclusive range of MMR positions for this segment
-	pub fn segment_pos_range(&self, last_pos: u64) -> (u64, u64) {
-		let segment_size = self.segment_unpruned_size(last_pos);
+	pub fn segment_pos_range(&self, mmr_size: u64) -> (u64, u64) {
+		let segment_size = self.segment_unpruned_size(mmr_size);
 		let leaf_offset = self.leaf_offset();
-		let first = pmmr::insertion_to_pmmr_index(leaf_offset + 1);
-		let last = if self.full_segment(last_pos) {
-			pmmr::insertion_to_pmmr_index(leaf_offset + segment_size)
+		let first = pmmr::insertion_to_pmmr_index(leaf_offset);
+		let last = if self.full_segment(mmr_size) {
+			pmmr::insertion_to_pmmr_index(leaf_offset + segment_size - 1)
 				+ (self.identifier.height as u64)
 		} else {
-			last_pos
+			mmr_size - 1
 		};
-
 		(first, last)
 	}
 
 	/// TODO - binary_search_by_key() here (can we assume these are sorted by pos?)
-	fn get_hash(&self, pos: u64) -> Result<Hash, SegmentError> {
+	fn get_hash(&self, pos0: u64) -> Result<Hash, SegmentError> {
 		self.hash_pos
 			.iter()
 			.zip(&self.hashes)
-			.find(|&(&p, _)| p == pos)
+			.find(|&(&p, _)| p == pos0)
 			.map(|(_, &h)| h)
-			.ok_or_else(|| SegmentError::MissingHash(pos))
+			.ok_or_else(|| SegmentError::MissingHash(pos0))
 	}
 
 	/// Get the identifier associated with this segment
@@ -178,16 +200,16 @@ impl<T> Segment<T> {
 		proof: SegmentProof,
 	) -> Self {
 		assert_eq!(hash_pos.len(), hashes.len());
-		let mut last_pos = 0;
+		let mut last = 0;
 		for &pos in &hash_pos {
-			assert!(pos > last_pos);
-			last_pos = pos;
+			assert!(last == 0 || pos > last);
+			last = pos;
 		}
 		assert_eq!(leaf_pos.len(), leaf_data.len());
-		last_pos = 0;
+		last = 0;
 		for &pos in &leaf_pos {
-			assert!(pos > last_pos);
-			last_pos = pos;
+			assert!(last == 0 || pos > last);
+			last = pos;
 		}
 
 		Self {
@@ -240,28 +262,28 @@ where
 	{
 		let mut segment = Segment::empty(segment_id);
 
-		let last_pos = pmmr.unpruned_size();
-		if segment.segment_unpruned_size(last_pos) == 0 {
+		let mmr_size = pmmr.unpruned_size();
+		if segment.segment_unpruned_size(mmr_size) == 0 {
 			return Err(SegmentError::NonExistent);
 		}
 
 		// Fill leaf data and hashes
-		let (segment_first_pos, segment_last_pos) = segment.segment_pos_range(last_pos);
-		for pos in segment_first_pos..=segment_last_pos {
-			if pmmr::is_leaf(pos) {
-				if let Some(data) = pmmr.get_data_from_file(pos) {
+		let (segment_first_pos, segment_last_pos) = segment.segment_pos_range(mmr_size);
+		for pos0 in segment_first_pos..=segment_last_pos {
+			if pmmr::is_leaf(pos0) {
+				if let Some(data) = pmmr.get_data_from_file(pos0) {
 					segment.leaf_data.push(data);
-					segment.leaf_pos.push(pos);
+					segment.leaf_pos.push(pos0);
 					continue;
 				} else if !prunable {
-					return Err(SegmentError::MissingLeaf(pos));
+					return Err(SegmentError::MissingLeaf(pos0));
 				}
 			}
 			// TODO: optimize, no need to send every intermediary hash
 			if prunable {
-				if let Some(hash) = pmmr.get_from_file(pos) {
+				if let Some(hash) = pmmr.get_from_file(pos0) {
 					segment.hashes.push(hash);
-					segment.hash_pos.push(pos);
+					segment.hash_pos.push(pos0);
 				}
 			}
 		}
@@ -269,12 +291,12 @@ where
 		let mut start_pos = None;
 		// Fully pruned segment: only include a single hash, the first unpruned parent
 		if segment.leaf_data.is_empty() && segment.hashes.is_empty() {
-			let family_branch = pmmr::family_branch(segment_last_pos, last_pos);
-			for (pos, _) in family_branch {
-				if let Some(hash) = pmmr.get_from_file(pos) {
+			let family_branch = pmmr::family_branch(segment_last_pos, mmr_size);
+			for (pos0, _) in family_branch {
+				if let Some(hash) = pmmr.get_from_file(pos0) {
 					segment.hashes.push(hash);
-					segment.hash_pos.push(pos);
-					start_pos = Some(pos);
+					segment.hash_pos.push(pos0);
+					start_pos = Some(1 + pos0);
 					break;
 				}
 			}
@@ -283,9 +305,9 @@ where
 		// Segment merkle proof
 		segment.proof = SegmentProof::generate(
 			pmmr,
-			last_pos,
-			segment_first_pos,
-			segment_last_pos,
+			mmr_size,
+			1 + segment_first_pos,
+			1 + segment_last_pos,
 			start_pos,
 		)?;
 
@@ -301,25 +323,25 @@ where
 	/// Returns `None` iff the segment is full and completely pruned
 	pub fn root(
 		&self,
-		last_pos: u64,
+		mmr_size: u64,
 		bitmap: Option<&Bitmap>,
 	) -> Result<Option<Hash>, SegmentError> {
-		let (segment_first_pos, segment_last_pos) = self.segment_pos_range(last_pos);
+		let (segment_first_pos, segment_last_pos) = self.segment_pos_range(mmr_size);
 		let mut hashes = Vec::<Option<Hash>>::with_capacity(2 * (self.identifier.height as usize));
-		let mut leaves = self.leaf_pos.iter().zip(&self.leaf_data);
-		for pos in segment_first_pos..=segment_last_pos {
-			let height = pmmr::bintree_postorder_height(pos);
+		let mut leaves0 = self.leaf_pos.iter().zip(&self.leaf_data);
+		for pos0 in segment_first_pos..=segment_last_pos {
+			let height = pmmr::bintree_postorder_height(pos0);
 			let hash = if height == 0 {
 				// Leaf
 				if bitmap
 					.map(|b| {
-						let idx_1 = pmmr::n_leaves(pos) - 1;
-						let idx_2 = if pmmr::is_left_sibling(pos) {
+						let idx_1 = pmmr::n_leaves(pos0 + 1) - 1;
+						let idx_2 = if pmmr::is_left_sibling(pos0) {
 							idx_1 + 1
 						} else {
 							idx_1 - 1
 						};
-						b.contains(idx_1 as u32) || b.contains(idx_2 as u32) || pos == last_pos
+						b.contains(idx_1 as u32) || b.contains(idx_2 as u32) || pos0 == mmr_size - 1
 					})
 					.unwrap_or(true)
 				{
@@ -329,17 +351,17 @@ where
 					//  require the last leaf to be present regardless of the status in the bitmap.
 					// TODO: possibly remove requirement on the sibling when we no longer support
 					//  syncing through the txhashset.zip method.
-					let data = leaves
-						.find(|&(&p, _)| p == pos)
+					let data = leaves0
+						.find(|&(&p, _)| p == pos0)
 						.map(|(_, l)| l)
-						.ok_or_else(|| SegmentError::MissingLeaf(pos))?;
-					Some(data.hash_with_index(pos - 1))
+						.ok_or_else(|| SegmentError::MissingLeaf(pos0))?;
+					Some(data.hash_with_index(pos0))
 				} else {
 					None
 				}
 			} else {
-				let left_child_pos = pos - (1 << height);
-				let right_child_pos = pos - 1;
+				let left_child_pos = 1 + pos0 - (1 << height);
+				let right_child_pos = pos0;
 
 				let right_child = hashes.pop().unwrap();
 				let left_child = hashes.pop().unwrap();
@@ -348,14 +370,14 @@ where
 					// Prunable MMR
 					match (left_child, right_child) {
 						(None, None) => None,
-						(Some(l), Some(r)) => Some((l, r).hash_with_index(pos - 1)),
+						(Some(l), Some(r)) => Some((l, r).hash_with_index(pos0)),
 						(None, Some(r)) => {
-							let l = self.get_hash(left_child_pos)?;
-							Some((l, r).hash_with_index(pos - 1))
+							let l = self.get_hash(left_child_pos - 1)?;
+							Some((l, r).hash_with_index(pos0))
 						}
 						(Some(l), None) => {
-							let r = self.get_hash(right_child_pos)?;
-							Some((l, r).hash_with_index(pos - 1))
+							let r = self.get_hash(right_child_pos - 1)?;
+							Some((l, r).hash_with_index(pos0))
 						}
 					}
 				} else {
@@ -366,70 +388,72 @@ where
 							right_child
 								.ok_or_else(|| SegmentError::MissingHash(right_child_pos))?,
 						)
-							.hash_with_index(pos - 1),
+							.hash_with_index(pos0),
 					)
 				}
 			};
 			hashes.push(hash);
 		}
 
-		if self.full_segment(last_pos) {
+		if self.full_segment(mmr_size) {
 			// Full segment: last position of segment is subtree root
 			Ok(hashes.pop().unwrap())
 		} else {
 			// Not full (only final segment): peaks in segment, bag them together
-			let peaks = pmmr::peaks(last_pos)
+			let peaks = pmmr::peaks(mmr_size)
 				.into_iter()
-				.filter(|&pos| pos >= segment_first_pos && pos <= segment_last_pos)
+				.filter(|&pos0| pos0 >= segment_first_pos && pos0 <= segment_last_pos)
 				.rev();
 			let mut hash = None;
-			for pos in peaks {
-				let mut lhash = hashes.pop().ok_or_else(|| SegmentError::MissingHash(pos))?;
+			for pos0 in peaks {
+				let mut lhash = hashes
+					.pop()
+					.ok_or_else(|| SegmentError::MissingHash(1 + pos0))?;
 				if lhash.is_none() && bitmap.is_some() {
 					// If this entire peak is pruned, load it from the segment hashes
-					lhash = Some(self.get_hash(pos)?);
+					lhash = Some(self.get_hash(pos0)?);
 				}
-				let lhash = lhash.ok_or_else(|| SegmentError::MissingHash(pos))?;
+				let lhash = lhash.ok_or_else(|| SegmentError::MissingHash(1 + pos0))?;
 
 				hash = match hash {
 					None => Some(lhash),
-					Some(rhash) => Some((lhash, rhash).hash_with_index(last_pos)),
+					Some(rhash) => Some((lhash, rhash).hash_with_index(mmr_size)),
 				};
 			}
 			Ok(Some(hash.unwrap()))
 		}
 	}
 
-	/// Get the first unpruned parent hash of this segment
+	/// Get the first 1-based (sucks) unpruned parent hash of this segment
 	pub fn first_unpruned_parent(
 		&self,
-		last_pos: u64,
+		mmr_size: u64,
 		bitmap: Option<&Bitmap>,
 	) -> Result<(Hash, u64), SegmentError> {
-		let root = self.root(last_pos, bitmap)?;
-		let (_, last) = self.segment_pos_range(last_pos);
+		let root = self.root(mmr_size, bitmap)?;
+		let (_, last) = self.segment_pos_range(mmr_size);
 		if let Some(root) = root {
-			return Ok((root, last));
+			return Ok((root, 1 + last));
 		}
 		let bitmap = bitmap.unwrap();
-		let n_leaves = pmmr::n_leaves(last_pos);
+		let n_leaves = pmmr::n_leaves(mmr_size);
 
 		let mut cardinality = 0;
-		let mut pos = last;
+		let mut pos0 = last;
 		let mut hash = Err(SegmentError::MissingHash(last));
-		let mut family_branch = pmmr::family_branch(last, last_pos).into_iter();
+		let mut family_branch = pmmr::family_branch(last, mmr_size).into_iter();
 		while cardinality == 0 {
-			hash = self.get_hash(pos).map(|h| (h, pos));
+			hash = self.get_hash(pos0).map(|h| (h, 1 + pos0));
 			if hash.is_ok() {
 				// Return early in case a lower level hash is already present
 				// This can occur if both child trees are pruned but compaction hasn't run yet
 				return hash;
 			}
 
-			if let Some((p, _)) = family_branch.next() {
-				pos = p;
-				let range = (pmmr::n_leaves(pmmr::bintree_leftmost(p)) - 1)
-					..min(pmmr::n_leaves(pmmr::bintree_rightmost(p)), n_leaves);
+			if let Some((p0, _)) = family_branch.next() {
+				pos0 = p0;
+				let range = (pmmr::n_leaves(1 + pmmr::bintree_leftmost(p0)) - 1)
+					..min(pmmr::n_leaves(1 + pmmr::bintree_rightmost(p0)), n_leaves);
 				cardinality = bitmap.range_cardinality(range);
 			} else {
 				break;
@@ -441,14 +465,14 @@ where
 	/// Check validity of the segment by calculating its root and validating the merkle proof
 	pub fn validate(
 		&self,
-		last_pos: u64,
+		mmr_size: u64,
 		bitmap: Option<&Bitmap>,
 		mmr_root: Hash,
 	) -> Result<(), SegmentError> {
-		let (first, last) = self.segment_pos_range(last_pos);
-		let (segment_root, segment_unpruned_pos) = self.first_unpruned_parent(last_pos, bitmap)?;
+		let (first, last) = self.segment_pos_range(mmr_size);
+		let (segment_root, segment_unpruned_pos) = self.first_unpruned_parent(mmr_size, bitmap)?;
 		self.proof.validate(
-			last_pos,
+			mmr_size,
 			mmr_root,
 			first,
 			last,
@@ -461,17 +485,17 @@ where
 	/// This function assumes a final hashing step together with `other_root`
 	pub fn validate_with(
 		&self,
-		last_pos: u64,
+		mmr_size: u64,
 		bitmap: Option<&Bitmap>,
 		mmr_root: Hash,
 		hash_last_pos: u64,
 		other_root: Hash,
 		other_is_left: bool,
 	) -> Result<(), SegmentError> {
-		let (first, last) = self.segment_pos_range(last_pos);
-		let (segment_root, segment_unpruned_pos) = self.first_unpruned_parent(last_pos, bitmap)?;
+		let (first, last) = self.segment_pos_range(mmr_size);
+		let (segment_root, segment_unpruned_pos) = self.first_unpruned_parent(mmr_size, bitmap)?;
 		self.proof.validate_with(
-			last_pos,
+			mmr_size,
 			mmr_root,
 			first,
 			last,
@@ -488,16 +512,16 @@ impl<T: Readable> Readable for Segment<T> {
 	fn read<R: Reader>(reader: &mut R) -> Result<Self, Error> {
 		let identifier = Readable::read(reader)?;
 
-		let mut last_pos = 0;
 		let n_hashes = reader.read_u64()? as usize;
 		let mut hash_pos = Vec::with_capacity(n_hashes);
+		let mut last_pos = 0;
 		for _ in 0..n_hashes {
 			let pos = reader.read_u64()?;
 			if pos <= last_pos {
 				return Err(Error::SortError);
 			}
 			last_pos = pos;
-			hash_pos.push(pos);
+			hash_pos.push(pos - 1);
 		}
 
 		let mut hashes = Vec::<Hash>::with_capacity(n_hashes);
@@ -514,7 +538,7 @@ impl<T: Readable> Readable for Segment<T> {
 				return Err(Error::SortError);
 			}
 			last_pos = pos;
-			leaf_pos.push(pos);
+			leaf_pos.push(pos - 1);
 		}
 
 		let mut leaf_data = Vec::<T>::with_capacity(n_leaves);
@@ -540,14 +564,14 @@ impl<T: Writeable> Writeable for Segment<T> {
 		Writeable::write(&self.identifier, writer)?;
 		writer.write_u64(self.hashes.len() as u64)?;
 		for &pos in &self.hash_pos {
-			writer.write_u64(pos)?;
+			writer.write_u64(1 + pos)?;
 		}
 		for hash in &self.hashes {
 			Writeable::write(hash, writer)?;
 		}
 		writer.write_u64(self.leaf_data.len() as u64)?;
 		for &pos in &self.leaf_pos {
-			writer.write_u64(pos)?;
+			writer.write_u64(1 + pos)?;
 		}
 		for data in &self.leaf_data {
 			Writeable::write(data, writer)?;
@@ -579,21 +603,24 @@ impl SegmentProof {
 		U: PMMRable,
 		B: Backend<U>,
 	{
-		let family_branch = pmmr::family_branch(segment_last_pos, last_pos);
+		let family_branch = pmmr::family_branch(segment_last_pos - 1, last_pos);
 
 		// 1. siblings along the path from the subtree root to the peak
 		let hashes: Result<Vec<_>, _> = family_branch
 			.iter()
-			.filter(|&&(p, _)| start_pos.map(|s| p > s).unwrap_or(true))
-			.map(|&(_, s)| pmmr.get_hash(s).ok_or_else(|| SegmentError::MissingHash(s)))
+			.filter(|&&(p0, _)| start_pos.map(|s| p0 >= s).unwrap_or(true))
+			.map(|&(_, s0)| {
+				pmmr.get_hash(s0)
+					.ok_or_else(|| SegmentError::MissingHash(s0))
+			})
 			.collect();
 		let mut proof = Self { hashes: hashes? };
 
 		// 2. bagged peaks to the right
 		let peak_pos = family_branch
 			.last()
-			.map(|&(p, _)| p)
-			.unwrap_or(segment_last_pos);
+			.map(|&(p0, _)| p0)
+			.unwrap_or(segment_last_pos - 1);
 		if let Some(h) = pmmr.bag_the_rhs(peak_pos) {
 			proof.hashes.push(h);
 		}
@@ -601,7 +628,7 @@ impl SegmentProof {
 		// 3. peaks to the left
 		let peaks: Result<Vec<_>, _> = pmmr::peaks(last_pos)
 			.into_iter()
-			.filter(|&x| x < segment_first_pos)
+			.filter(|&x| 1 + x < segment_first_pos)
 			.rev()
 			.map(|p| pmmr.get_hash(p).ok_or_else(|| SegmentError::MissingHash(p)))
 			.collect();
@@ -619,43 +646,46 @@ impl SegmentProof {
 	pub fn reconstruct_root(
 		&self,
 		last_pos: u64,
-		segment_first_pos: u64,
-		segment_last_pos: u64,
+		segment_first_pos0: u64,
+		segment_last_pos0: u64,
 		segment_root: Hash,
 		segment_unpruned_pos: u64,
 	) -> Result<Hash, SegmentError> {
 		let mut iter = self.hashes.iter();
-		let family_branch = pmmr::family_branch(segment_last_pos, last_pos);
+		let family_branch = pmmr::family_branch(segment_last_pos0, last_pos);
 
 		// 1. siblings along the path from the subtree root to the peak
 		let mut root = segment_root;
-		for &(p, s) in family_branch
+		for &(p0, s0) in family_branch
 			.iter()
-			.filter(|&&(p, _)| p > segment_unpruned_pos)
+			.filter(|&&(p0, _)| p0 >= segment_unpruned_pos)
 		{
-			let sibling_hash = iter.next().ok_or_else(|| SegmentError::MissingHash(s))?;
-			root = if pmmr::is_left_sibling(s) {
-				(sibling_hash, root).hash_with_index(p - 1)
+			let sibling_hash = iter
+				.next()
+				.ok_or_else(|| SegmentError::MissingHash(1 + s0))?;
+			root = if pmmr::is_left_sibling(s0) {
+				(sibling_hash, root).hash_with_index(p0)
 			} else {
-				(root, sibling_hash).hash_with_index(p - 1)
+				(root, sibling_hash).hash_with_index(p0)
 			};
 		}
 
 		// 2. bagged peaks to the right
-		let peak_pos = family_branch
+		let peak_pos0 = family_branch
 			.last()
-			.map(|&(p, _)| p)
-			.unwrap_or(segment_last_pos);
+			.map(|&(p0, _)| p0)
+			.unwrap_or(segment_last_pos0);
 
 		let rhs = pmmr::peaks(last_pos)
 			.into_iter()
-			.filter(|&x| x > peak_pos)
+			.filter(|&x| x > peak_pos0)
 			.next();
 
-		if let Some(pos) = rhs {
+		if let Some(pos0) = rhs {
 			root = (
 				root,
-				iter.next().ok_or_else(|| SegmentError::MissingHash(pos))?,
+				iter.next()
+					.ok_or_else(|| SegmentError::MissingHash(1 + pos0))?,
 			)
 				.hash_with_index(last_pos)
 		}
@@ -663,11 +693,12 @@ impl SegmentProof {
 		// 3. peaks to the left
 		let peaks = pmmr::peaks(last_pos)
 			.into_iter()
-			.filter(|&x| x < segment_first_pos)
+			.filter(|&x| x < segment_first_pos0)
 			.rev();
-		for pos in peaks {
+		for pos0 in peaks {
 			root = (
-				iter.next().ok_or_else(|| SegmentError::MissingHash(pos))?,
+				iter.next()
+					.ok_or_else(|| SegmentError::MissingHash(1 + pos0))?,
 				root,
 			)
 				.hash_with_index(last_pos);
