@@ -1,4 +1,4 @@
-// Copyright 2020 The Grin Developers
+// Copyright 2021 The Grin Developers
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -27,10 +27,8 @@ extern crate enum_primitive;
 extern crate lazy_static;
 #[macro_use]
 extern crate serde_derive;
-use serde;
 #[macro_use]
 extern crate log;
-use failure;
 #[macro_use]
 extern crate failure_derive;
 #[macro_use]
