@@ -1,4 +1,4 @@
-// Copyright 2020 The Grin Developers
+// Copyright 2021 The Grin Developers
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -41,6 +41,6 @@ pub fn tx_fee(input_len: usize, output_len: usize, kernel_len: usize) -> u64 {
 }
 
 /// Transaction fee calculation given transaction
-pub fn accept_fee(tx: Transaction, height: u64) -> u64 {
-	tx.accept_fee(height)
+pub fn accept_fee(tx: Transaction) -> u64 {
+	tx.accept_fee()
 }
