@@ -1,4 +1,4 @@
-// Copyright 2020 The Grin Developers
+// Copyright 2021 The Grin Developers
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ pub mod common;
 use self::core::core::block::BlockHeader;
 use self::core::core::block::Error::KernelLockHeight;
 use self::core::core::hash::{Hashed, ZERO_HASH};
-use self::core::core::verifier_cache::{LruVerifierCache, VerifierCache};
 use self::core::core::{
 	aggregate, deaggregate, FeeFields, KernelFeatures, Output, OutputFeatures, OutputIdentifier,
 	Transaction, TxKernel, Weighting,
@@ -30,9 +29,7 @@ use self::core::{global, ser};
 use crate::common::{new_block, tx1i1o, tx1i2o, tx2i1o};
 use grin_core as core;
 use keychain::{BlindingFactor, ExtKeychain, Keychain};
-use std::sync::Arc;
 use util::static_secp_instance;
-use util::RwLock;
 
 // Setup test with AutomatedTesting chain_type;
 fn test_setup() {
@@ -97,8 +94,7 @@ fn simple_tx_ser_deser() {
 	let mut vec = Vec::new();
 	ser::serialize_default(&mut vec, &tx).expect("serialization failed");
 	let dtx: Transaction = ser::deserialize_default(&mut &vec[..]).unwrap();
-	let height = 42; // arbitrary
-	assert_eq!(dtx.fee(height), 2);
+	assert_eq!(dtx.fee(), 2);
 	assert_eq!(dtx.inputs().len(), 2);
 	assert_eq!(dtx.outputs().len(), 1);
 	assert_eq!(tx.hash(), dtx.hash());
@@ -141,10 +137,6 @@ fn test_zero_commit_fails() {
 	assert!(res.is_err());
 }
 
-fn verifier_cache() -> Arc<RwLock<dyn VerifierCache>> {
-	Arc::new(RwLock::new(LruVerifierCache::new()))
-}
-
 #[test]
 fn build_tx_kernel() {
 	test_setup();
@@ -164,9 +156,7 @@ fn build_tx_kernel() {
 	.unwrap();
 
 	// check the tx is valid
-	let height = 42; // arbitrary
-	tx.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.unwrap();
+	tx.validate(Weighting::AsTransaction).unwrap();
 
 	// check the kernel is also itself valid
 	assert_eq!(tx.kernels().len(), 1);
@@ -174,7 +164,7 @@ fn build_tx_kernel() {
 	kern.verify().unwrap();
 
 	assert_eq!(kern.features, KernelFeatures::Plain { fee: 2.into() });
-	assert_eq!(2, tx.fee(height));
+	assert_eq!(2, tx.fee());
 }
 
 // Proof of concept demonstrating we can build two transactions that share
@@ -228,16 +218,9 @@ fn build_two_half_kernels() {
 	)
 	.unwrap();
 
-	let height = 42; // arbitrary
-	assert_eq!(
-		tx1.validate(Weighting::AsTransaction, verifier_cache(), height),
-		Ok(()),
-	);
+	assert_eq!(tx1.validate(Weighting::AsTransaction), Ok(()),);
 
-	assert_eq!(
-		tx2.validate(Weighting::AsTransaction, verifier_cache(), height),
-		Ok(()),
-	);
+	assert_eq!(tx2.validate(Weighting::AsTransaction), Ok(()),);
 
 	// The transactions share an identical kernel.
 	assert_eq!(tx1.kernels()[0], tx2.kernels()[0]);
@@ -261,22 +244,14 @@ fn transaction_cut_through() {
 	let tx1 = tx1i2o();
 	let tx2 = tx2i1o();
 
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.is_ok());
 
-	let vc = verifier_cache();
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
 
 	// now build a "cut_through" tx from tx1 and tx2
 	let tx3 = aggregate(&[tx1, tx2]).unwrap();
 
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
 }
 
 // Attempt to deaggregate a multi-kernel transaction in a different way
@@ -288,46 +263,30 @@ fn multi_kernel_transaction_deaggregation() {
 	let tx3 = tx1i1o();
 	let tx4 = tx1i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx4
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx4.validate(Weighting::AsTransaction).is_ok());
 
 	let tx1234 = aggregate(&[tx1.clone(), tx2.clone(), tx3.clone(), tx4.clone()]).unwrap();
 	let tx12 = aggregate(&[tx1, tx2]).unwrap();
 	let tx34 = aggregate(&[tx3, tx4]).unwrap();
 
-	assert!(tx1234
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx12
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx34
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1234.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx12.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx34.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx34 = deaggregate(tx1234.clone(), &[tx12.clone()]).unwrap();
 	assert!(deaggregated_tx34
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
+
 		.is_ok());
 	assert_eq!(tx34, deaggregated_tx34);
 
 	let deaggregated_tx12 = deaggregate(tx1234, &[tx34]).unwrap();
 
 	assert!(deaggregated_tx12
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx12, deaggregated_tx12);
 }
@@ -339,32 +298,19 @@ fn multi_kernel_transaction_deaggregation_2() {
 	let tx2 = tx1i1o();
 	let tx3 = tx1i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
 
 	let tx123 = aggregate(&[tx1.clone(), tx2.clone(), tx3.clone()]).unwrap();
 	let tx12 = aggregate(&[tx1, tx2]).unwrap();
 
-	assert!(tx123
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx12
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx123.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx12.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx3 = deaggregate(tx123, &[tx12]).unwrap();
 	assert!(deaggregated_tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx3, deaggregated_tx3);
 }
@@ -376,33 +322,20 @@ fn multi_kernel_transaction_deaggregation_3() {
 	let tx2 = tx1i1o();
 	let tx3 = tx1i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
 
 	let tx123 = aggregate(&[tx1.clone(), tx2.clone(), tx3.clone()]).unwrap();
 	let tx13 = aggregate(&[tx1, tx3]).unwrap();
 	let tx2 = aggregate(&[tx2]).unwrap();
 
-	assert!(tx123
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx123.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx13 = deaggregate(tx123, &[tx2]).unwrap();
 	assert!(deaggregated_tx13
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx13, deaggregated_tx13);
 }
@@ -416,24 +349,11 @@ fn multi_kernel_transaction_deaggregation_4() {
 	let tx4 = tx1i1o();
 	let tx5 = tx1i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx4
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx5
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx4.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx5.validate(Weighting::AsTransaction).is_ok());
 
 	let tx12345 = aggregate(&[
 		tx1.clone(),
@@ -443,13 +363,11 @@ fn multi_kernel_transaction_deaggregation_4() {
 		tx5.clone(),
 	])
 	.unwrap();
-	assert!(tx12345
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx12345.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx5 = deaggregate(tx12345, &[tx1, tx2, tx3, tx4]).unwrap();
 	assert!(deaggregated_tx5
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx5, deaggregated_tx5);
 }
@@ -463,24 +381,11 @@ fn multi_kernel_transaction_deaggregation_5() {
 	let tx4 = tx1i1o();
 	let tx5 = tx1i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx4
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx5
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx4.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx5.validate(Weighting::AsTransaction).is_ok());
 
 	let tx12345 = aggregate(&[
 		tx1.clone(),
@@ -493,13 +398,11 @@ fn multi_kernel_transaction_deaggregation_5() {
 	let tx12 = aggregate(&[tx1, tx2]).unwrap();
 	let tx34 = aggregate(&[tx3, tx4]).unwrap();
 
-	assert!(tx12345
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx12345.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx5 = deaggregate(tx12345, &[tx12, tx34]).unwrap();
 	assert!(deaggregated_tx5
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx5, deaggregated_tx5);
 }
@@ -511,34 +414,25 @@ fn basic_transaction_deaggregation() {
 	let tx1 = tx1i2o();
 	let tx2 = tx2i1o();
 
-	let vc = verifier_cache();
-
-	let height = 42; // arbitrary
-	assert!(tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
-	assert!(tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx1.validate(Weighting::AsTransaction).is_ok());
+	assert!(tx2.validate(Weighting::AsTransaction).is_ok());
 
 	// now build a "cut_through" tx from tx1 and tx2
 	let tx3 = aggregate(&[tx1.clone(), tx2.clone()]).unwrap();
 
-	assert!(tx3
-		.validate(Weighting::AsTransaction, vc.clone(), height)
-		.is_ok());
+	assert!(tx3.validate(Weighting::AsTransaction).is_ok());
 
 	let deaggregated_tx1 = deaggregate(tx3.clone(), &[tx2.clone()]).unwrap();
 
 	assert!(deaggregated_tx1
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx1, deaggregated_tx1);
 
 	let deaggregated_tx2 = deaggregate(tx3, &[tx1]).unwrap();
 
 	assert!(deaggregated_tx2
-		.validate(Weighting::AsTransaction, vc.clone(), height)
+		.validate(Weighting::AsTransaction)
 		.is_ok());
 	assert_eq!(tx2, deaggregated_tx2);
 }
@@ -568,10 +462,7 @@ fn hash_output() {
 #[test]
 fn blind_tx() {
 	let btx = tx2i1o();
-	let height = 42; // arbitrary
-	assert!(btx
-		.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.is_ok());
+	assert!(btx.validate(Weighting::AsTransaction).is_ok());
 
 	// Ignored for bullet proofs, because calling range_proof_info
 	// with a bullet proof causes painful errors
@@ -641,10 +532,7 @@ fn tx_build_exchange() {
 	)
 	.unwrap();
 
-	let height = 42; // arbitrary
-	tx_final
-		.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.unwrap();
+	tx_final.validate(Weighting::AsTransaction).unwrap();
 }
 
 #[test]
@@ -658,8 +546,7 @@ fn reward_empty_block() {
 
 	let b = new_block(&[], &keychain, &builder, &previous_header, &key_id);
 
-	b.validate(&BlindingFactor::zero(), verifier_cache())
-		.unwrap();
+	b.validate(&BlindingFactor::zero()).unwrap();
 }
 
 #[test]
@@ -669,19 +556,13 @@ fn reward_with_tx_block() {
 	let builder = ProofBuilder::new(&keychain);
 	let key_id = ExtKeychain::derive_key_id(1, 1, 0, 0, 0);
 
-	let vc = verifier_cache();
-
 	let tx1 = tx2i1o();
 	let previous_header = BlockHeader::default();
-	tx1.validate(
-		Weighting::AsTransaction,
-		vc.clone(),
-		previous_header.height + 1,
-	)
-	.unwrap();
+	tx1.validate(Weighting::AsTransaction)
+		.unwrap();
 
 	let block = new_block(&[tx1], &keychain, &builder, &previous_header, &key_id);
-	block.validate(&BlindingFactor::zero(), vc.clone()).unwrap();
+	block.validate(&BlindingFactor::zero()).unwrap();
 }
 
 #[test]
@@ -691,15 +572,13 @@ fn simple_block() {
 	let builder = ProofBuilder::new(&keychain);
 	let key_id = ExtKeychain::derive_key_id(1, 1, 0, 0, 0);
 
-	let vc = verifier_cache();
-
 	let tx1 = tx2i1o();
 	let tx2 = tx1i1o();
 
 	let previous_header = BlockHeader::default();
 	let b = new_block(&[tx1, tx2], &keychain, &builder, &previous_header, &key_id);
 
-	b.validate(&BlindingFactor::zero(), vc.clone()).unwrap();
+	b.validate(&BlindingFactor::zero()).unwrap();
 }
 
 #[test]
@@ -711,8 +590,6 @@ fn test_block_with_timelocked_tx() {
 	let key_id2 = ExtKeychain::derive_key_id(1, 2, 0, 0, 0);
 	let key_id3 = ExtKeychain::derive_key_id(1, 3, 0, 0, 0);
 
-	let vc = verifier_cache();
-
 	// first check we can add a timelocked tx where lock height matches current
 	// block height and that the resulting block is valid
 	let tx1 = build::transaction(
@@ -735,7 +612,7 @@ fn test_block_with_timelocked_tx() {
 		&previous_header,
 		&key_id3.clone(),
 	);
-	b.validate(&BlindingFactor::zero(), vc.clone()).unwrap();
+	b.validate(&BlindingFactor::zero()).unwrap();
 
 	// now try adding a timelocked tx where lock height is greater than current
 	// block height
@@ -753,7 +630,7 @@ fn test_block_with_timelocked_tx() {
 	let previous_header = BlockHeader::default();
 	let b = new_block(&[tx1], &keychain, &builder, &previous_header, &key_id3);
 
-	match b.validate(&BlindingFactor::zero(), vc.clone()) {
+	match b.validate(&BlindingFactor::zero()) {
 		Err(KernelLockHeight(height)) => {
 			assert_eq!(height, 2);
 		}
@@ -765,16 +642,12 @@ fn test_block_with_timelocked_tx() {
 pub fn test_verify_1i1o_sig() {
 	test_setup();
 	let tx = tx1i1o();
-	let height = 42; // arbitrary
-	tx.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.unwrap();
+	tx.validate(Weighting::AsTransaction).unwrap();
 }
 
 #[test]
 pub fn test_verify_2i1o_sig() {
 	test_setup();
 	let tx = tx2i1o();
-	let height = 42; // arbitrary
-	tx.validate(Weighting::AsTransaction, verifier_cache(), height)
-		.unwrap();
+	tx.validate(Weighting::AsTransaction).unwrap();
 }
